Package: MCSimMod
Title: Working with 'MCSim' Models
Version: 1.2
Authors@R: c(
  person("Dustin F.", "Kapraun", role=c("aut", "cre"), email = "kapraun.dustin@epa.gov", comment = c(ORCID = "0000-0001-5570-6383")),
  person("Todd J.", "Zurlinden", role="aut", comment = c(ORCID = "0000-0003-1372-3913")),
  person("Andrew J.", "Shapiro", role="aut", comment = c(ORCID = "0000-0002-5233-8092")),
  person("Ryan D.", "Friese", role="aut", comment = c(ORCID = "0000-0002-4121-2195")),
  person("Frederic Y.", "Bois", role="ctb", comment = c(ORCID = " 0000-0002-4154-0391")),
  person("Free Software Foundation, Inc.", role="cph")
  )
Description: Tools that facilitate ordinary differential equation (ODE) modeling in 'R'. This package allows one to perform simulations for ODE models that are encoded in the GNU 'MCSim' model specification language (Bois, 2009) <doi:10.1093/bioinformatics/btp162> using ODE solvers from the 'R' package 'deSolve' (Soetaert et al., 2010) <doi:10.18637/jss.v033.i09>.
Imports: deSolve, methods, tools
URL: https://CRAN.R-project.org/package=MCSimMod,
        https://github.com/USEPA/MCSimMod
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
Config/Needs/dev: devtools, styler (== 1.11.0), testthat, covr
Config/Needs/website: r-lib/pkgdown
Config/testthat/edition: 3
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2026-01-22 22:02:42 UTC; dkapraun
Author: Dustin F. Kapraun [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5570-6383>),
  Todd J. Zurlinden [aut] (ORCID:
    <https://orcid.org/0000-0003-1372-3913>),
  Andrew J. Shapiro [aut] (ORCID:
    <https://orcid.org/0000-0002-5233-8092>),
  Ryan D. Friese [aut] (ORCID: <https://orcid.org/0000-0002-4121-2195>),
  Frederic Y. Bois [ctb] (ORCID: <https://orcid.org/0000-0002-4154-0391>),
  Free Software Foundation, Inc. [cph]
Maintainer: Dustin F. Kapraun <kapraun.dustin@epa.gov>
Repository: CRAN
Date/Publication: 2026-01-23 13:40:13 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 20:16:14 UTC; unix
Archs: MCSimMod.so.dSYM
